import argparse

class Preferences:
    ''' Manage user-specific settings based on command line arguments '''

    def __init__(self):

        parser = self._configure_parser()
        args = parser.parse_args()

        for arg, value in vars(args).items():
            setattr(self, arg, value)
        self.number = self.size**2

    def _configure_parser(self) -> argparse.ArgumentParser:
        ''' Set up argparse and provide the necessary info for a help page '''

        parser = argparse.ArgumentParser(
            prog='lbcollage',
            description='Create a collage of album art from your ListenBrainz listening history',
        )

        parser.add_argument(
            'username', 
            help='Your ListenBrainz username',
        )

        parser.add_argument(
            '-t', '--timespan', 
            metavar='timespan',
            help=('What timespan to consider - week, month, year, or all_time (default month)'),
            choices=['week', 'month', 'year', 'all_time'],
            default='month',
            type=str,
        )

        parser.add_argument(
            '-s', '--size', 
            metavar='size',
            help=('Choose what size the collage will be - the side length of the square'
                  '(default 3)'),
            default=3,
            type=int,
        )

        parser.add_argument(
            '-d', '--details', 
            #metavar='details',
            help='Print the album details on the covers',
            default=False,
            action='store_true',
        )

        parser.add_argument(
            '-o', '--output', 
            metavar='output',
            help='Set a custom output location for the collage image file',
            default=None,
            type=str,
        )

        return parser



import io
from PIL import Image, ImageDraw, ImageFont
import datetime
import os

from .preferences import Preferences
from .constants import IMAGE_SIZE

class ImageProcessing:
    ''' Turn the album art from MusicBrainz into a collage and save it to a file '''

    def create_collage(
        self, 
        prefs: Preferences, 
        images: list[bytes], 
        releases: list[str],
    ) -> bytes:
        ''' Compile the images from bytes with pillow '''
        
        print('Creating the collage...')
        collage = Image.new('RGB', [prefs.size*IMAGE_SIZE]*2)

        for image, release, position in zip(images, releases, self._image_positions(prefs)):
            
            collage.paste(
                (Image.open(io.BytesIO(image)) if image 
                 else Image.new('RGB', [IMAGE_SIZE]*2, (0, 0, 0))),
                position,
            )

            if prefs.details: self._add_details(collage, release, position)

        return collage
            
    def _image_positions(self, prefs: Preferences) -> tuple[int, int]:
        ''' Yield the position each image should be in the collage in series '''

        x, y = 0, 0
        for row in range(prefs.size):
            for column in range(prefs.size):
                yield x, y
                x += IMAGE_SIZE
            y += IMAGE_SIZE
            x = 0

    def _pad_position(self, position: tuple[int, int], padding=0.025):
        ''' Adding padding to image positions for printing text details '''

        return tuple([pos+padding*IMAGE_SIZE for pos in position])

    def _add_details(self, image: Image, release: dict, position: tuple[int, int]):
        ''' Print the release details on the cover if requested '''

        draw = ImageDraw.Draw(image)
        draw.text(
            self._pad_position(position), 
            '{release_name}\n{artist_name}'.format(**release), 
            (255, 255, 255), 
            font=ImageFont.truetype('fonts/CascadiaCode-Bold.ttf', 10),
        )

    def save_collage(self, prefs: Preferences, collage: Image):
        ''' Save the collage to a file, stamped with some preferences and the date '''

        date_string = datetime.datetime.now().strftime('%d%b%Y')
        filename = f'lbcollage_{prefs.username}_{prefs.timespan}_{date_string}.jpg'
        collage.save(os.path.join(prefs.output, filename) if prefs.output else filename)
        print(f'Successfully saved as \'{filename}\'!')




from .preferences import Preferences
from .brainzrequest import BrainzRequest
from .imageprocessing import ImageProcessing

def main():

    IMAGE_SIZE = 250

    # Configure user preferences
    prefs = Preferences()

    # Request the necessary data from ListenBrainz and MusicBrainz
    brainz_request = BrainzRequest(prefs)

    # Compile the album art into a collage
    image_processing = ImageProcessing()
    collage = image_processing.create_collage(
        prefs, 
        brainz_request.cover_images, 
        brainz_request.releases,
    )

    # Save the collage to a local file
    image_processing.save_collage(prefs, collage)




import pylistenbrainz
import musicbrainzngs

from .preferences import Preferences
from .constants import IMAGE_SIZE

class BrainzRequest:
    ''' Request and store user and image data from ListenBrainz and MusicBrainz '''

    def __init__(self, prefs: Preferences):

        self.lb_client = pylistenbrainz.ListenBrainz()

        print(f'Getting listening data for {prefs.username}...')
        self.releases = self._get_user_album_info(self.lb_client, prefs)
        if len(self.releases) != prefs.number:
            raise ValueError('ListenBrainz didn\' return enough data to proceed with!')

        print('Downloading the album art...')
        self.cover_images = [self._get_album_cover(release['release_mbid']) 
                             for release in self.releases]

    def _get_user_album_info(
        self, 
        client: pylistenbrainz.client.ListenBrainz, 
        prefs: Preferences
    ) -> dict:
        ''' Find the user's most listened albums in the given timespan '''

        response = client.get_user_releases(
            username=prefs.username,
            time_range=prefs.timespan,
            count=prefs.number,
        )
        releases = response['payload']['releases']

        return releases

    def _get_album_cover(self, mbid: str) -> bytes | None:
        ''' Download an album cover given a MusicBrainz ID (mbid) '''

        try:
            cover = musicbrainzngs.get_image_front(mbid, size=IMAGE_SIZE)
            return cover
        except musicbrainzngs.ResponseError as e:
            return None
